﻿using Azure.Storage.Blobs;

using Microsoft.Extensions.Logging;

using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Schema;

using System;
using System.Collections.Generic;
using System.IO;

using TemplateContent.Common.Models;

namespace TemplateContent.Common
{
    internal class SchemaHelper
    {
        ILogger _logger;

        internal SchemaHelper(ILogger logger)
        {
            _logger = logger;
        }

        /// <summary>
        /// Gets a blob from Azure Blob storage. Those blobs should contain JSON schemas.
        /// </summary>
        /// <param name="schemaContentPath"></param>
        /// <returns></returns>
        internal string GetSchemaContent(string schemaContentPath)
        {
            if (string.IsNullOrEmpty(schemaContentPath))
                throw new ArgumentNullException(nameof(schemaContentPath));

            string result = string.Empty;

            BlobClient blobClient = new BlobClient(new Uri(schemaContentPath));
            var schemaBlobDownloadResponse = blobClient.Download();

            if (schemaBlobDownloadResponse != null && schemaBlobDownloadResponse.Value != null)
            {
                using (var downLoadInfo = schemaBlobDownloadResponse.Value)
                using (var streamedContent = new StreamReader(downLoadInfo.Content))
                {
                    result = streamedContent.ReadToEnd();
                }
            }

            return result;
        }

        /// <summary>
        /// Checks if a JSON message respects a given JSON Schema.
        /// </summary>
        /// <param name="message"></param>
        /// <param name="schema"></param>
        /// <returns></returns>
        internal SchemaValidationResponse CheckSchemaMessage(string message, string schema)
        {
            if (string.IsNullOrEmpty(message))
                throw new ArgumentNullException(nameof(message));

            if (string.IsNullOrEmpty(schema))
                throw new ArgumentNullException(nameof(schema));

            SchemaValidationResponse result = new SchemaValidationResponse { Message = message, IsMessageValid = false };

            try
            {

                JSchemaUrlResolver resolver = new JSchemaUrlResolver();
                JSchema jSchema = JSchema.Parse(schema, resolver);
                JObject jsonMessage = JObject.Parse(message);

                IList<string> errors = null;
                if (!jsonMessage.IsValid(jSchema, out errors))
                {
                    if (errors != null && errors.Count > 0)
                    {
                        result.Errors = errors;
                    }
                }
                else
                {
                    result.IsMessageValid = true;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError($"CheckSchema - Error {ex.Message}");
            }


            return result;
        }
    }
}
