using Azure.Messaging.ServiceBus;

using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.SignalRService;
using Microsoft.Extensions.Logging;

using Newtonsoft.Json.Linq;

using System;
using System.Text;
using System.Threading.Tasks;

using TemplateContent.Common;

namespace TemplateContent.IngressProcessor
{
    public static class QueueIngressProcessorController
    {
        private static ILogger _logger;

        [FunctionName("QueueIngressProcessor")]
        public static async Task Run(
            [ServiceBusTrigger("%afBindingInputQueueName%", Connection = "afBindingInputQueueConnectionString")] ServiceBusReceivedMessage iotHubMessage,
            [SignalR(HubName = "%afBindingSRHub%")] IAsyncCollector<SignalRMessage> signalRMessages,
            ILogger logger)
        {
            if (iotHubMessage == null)
                throw new ArgumentNullException(nameof(iotHubMessage));

            if (logger == null)
                throw new ArgumentNullException(nameof(logger));

            _logger = logger;

            string messageContent = Encoding.UTF8.GetString(iotHubMessage.Body);
            string deviceId = iotHubMessage.ApplicationProperties["iothub-connection-device-id"].ToString();

            _logger.LogDebug($"SCHEMA CONTROL for message-deviceID:{deviceId}-Message:{messageContent}");

            JObject jIoTHubMessage = JObject.Parse(messageContent);

            try
            {
                if (jIoTHubMessage.TryGetValue("$schema", out JToken schemaPath))
                {
                    SchemaHelper schemaHelper = new SchemaHelper(_logger);

                    //Message schema control
                    string schemaContent = schemaHelper.GetSchemaContent(schemaPath.Value<string>());

                    string messageType = string.Empty;

                    if (jIoTHubMessage.ContainsKey("messageType"))
                        messageType = jIoTHubMessage["messageType"].Value<string>();

                    var result = schemaHelper.CheckSchemaMessage(messageContent, schemaContent);

                    if (result != null && result.IsMessageValid)
                    {
                        _logger.LogDebug($"   >>>SCHEMA OK::{deviceId}::Message::{Environment.NewLine}{messageContent}");

                        //Add commissioning control at this level
                        string? checkCommissioning = Environment.GetEnvironmentVariable("checkCommissioning");
                        if (bool.TryParse(checkCommissioning, out bool checkCommissioningBool) && checkCommissioningBool)
                        {
                            _logger.LogDebug("   >>>COMMISSIONING CHECK");

                            if (await CommissioningHelper.IsDeviceCommissioned(deviceId, _logger))
                                await OutputsHelper.ProcessOutputs(iotHubMessage, deviceId, messageType, signalRMessages, _logger);
                            else
                                _logger.LogWarning($"   >>>DEVICE {deviceId} NOT COMMISSIONED");

                        }
                        else
                        {
                            _logger.LogDebug("   >>>NO COMMISSIONING CHECK");
                            await OutputsHelper.ProcessOutputs(iotHubMessage, deviceId, messageType, signalRMessages, _logger);
                        }
                    }
                    else
                    {
                        string errorMessage = String.Empty;
                        if (result.Errors != null)
                            errorMessage = string.Join(Environment.NewLine, result.Errors);

                        _logger.LogDebug($"   >>>SCHEMA KO::{deviceId}::Message::{Environment.NewLine}{messageContent}::{Environment.NewLine}Errors::{Environment.NewLine}{errorMessage}");

                        if (bool.Parse(Environment.GetEnvironmentVariable("afEnableSchemaErrorsStorage")))
                            await SchemaErrorsStorageHelper.ProcessSchemaErrors(_logger, result, deviceId, String.Empty, messageType);
                    }
                }
                else
                {
                    _logger.LogDebug($"   >>>NO SCHEMA PROVIDED.THE MESSAGE CANNOT BE ACCEPTED::{deviceId}::{Environment.NewLine}{messageContent}.");
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex.Message);
            }
        }
    }
}
